package service

import (
	"fmt"
	aModel "go-admin/app/admin/models"
	"go-admin/app/observe/models"
	"go-admin/app/observe/service/dto"
	cDto "go-admin/common/dto"
	"go-admin/utils"
	"time"

	"github.com/pkg/errors"
	"gorm.io/gorm"
)

type Event struct {
	utils.OtService
}

func (e *Event) GetPage(c *dto.EventListReq, result *[]dto.EventListResp, count *int64) error {
	db := e.ChOrm.Table(models.TableNameEvent)
	if c.AppId > 0 {
		db.Where("AppId", c.AppId)
	}
	if c.AlertStatus == 2 || c.AlertStatus == 4 {
		db.Where("AlertStatus", c.AlertStatus)
	}
	// db.Scopes(cDto.MakeCondition(*c))
	if c.StartTime > 0 {
		db.Where("AppendTime>=?", c.StartTime)
	}
	if c.EndTime > 0 {
		db.Where("AppendTime<?", c.EndTime)
	}
	if c.ExceptionName != "" {
		db.Where("ExceptionName LIKE ?", fmt.Sprintf("%%%s%%", c.ExceptionName))
	}
	err := db.Debug().Scopes(cDto.Paginate(c.GetPageSize(), c.GetPageIndex())).Order("AppendTime DESC").Find(result).
		Limit(-1).Offset(-1).Count(count).Error
	if err != nil {
		if err == gorm.ErrRecordNotFound {
			return nil
		}
		return errors.Wrap(err, "查询事件信息列表失败")
	}
	return nil
}

func (e *Event) ExceptionNums(c *dto.EventExceptionNumsReq, result *[]dto.EventExceptionNumsResp, count *int64) error {
	tbl := aModel.OtFireEvents{}.TableName()
	db := e.Orm.Table(fmt.Sprintf("%s t", tbl)).Scopes(
		cDto.Paginate(c.GetPageSize(), c.GetPageIndex()),
	)
	db.Select([]string{
		"app_id",
		fmt.Sprintf("(SELECT app_name FROM %s WHERE app_id=t.app_id ORDER BY created_at DESC LIMIT 1) AS app_name", tbl),
		"COUNT(1) AS exception_num",
	})
	db.Where("is_know = 0 AND is_resolve = 0 AND is_ignore = 0")
	if c.StartTime > 0 {
		db.Where("created_at>=?", time.Unix(c.StartTime, 0).Format(time.DateTime))
	}
	if c.EndTime > 0 {
		db.Where("created_at<?", time.Unix(c.EndTime, 0).Format(time.DateTime))
	}
	appAliases := c.GetAppAliases()
	if len(appAliases) > 0 {
		db.Where("app_alias in ?", appAliases)
	} else { // 兼容旧逻辑， 后期去掉
		appIds := c.GetAppIds()
		if len(appIds) > 0 {
			db.Where("app_id IN ?", appIds)
		}
	}
	if err := db.Group("app_id").Find(result).Count(count).Error; err != nil {
		return errors.Wrap(err, "执行查询应用异常数量失败")
	}
	return nil
}

func (e *Event) ExceptionNumByID(c *dto.EventExecptionNumGetByIDReq, count *int64) error {
	tbl := aModel.OtFireEvents{}.TableName()
	db := e.Orm.Table(fmt.Sprintf("%s t", tbl))
	db.Select([]string{
		"app_id",
		"COUNT(*) AS exception_num",
	})
	db.Where("app_id = ?", c.Id)
	db.Where("is_know = 0 AND is_resolve = 0 AND is_ignore = 0")
	if err := db.Count(count).Error; err != nil {
		return errors.Wrap(err, "执行查询业务异常数量失败")
	}
	return nil
}

func (e *Event) EventStatistic(c *dto.EventStatisticReq, result *[]dto.EventStatisticResp) error {
	db := e.ChOrm.Table(models.TableNameTrace)
	db.Select([]string{
		"toTimeZone(toStartOfFiveMinutes(Timestamp), 'Asia/Shanghai') AS FiveMinuteInterval",
		"COUNT(*) AS StatusMessageCount",
	})
	if c.AppAlias != "" {
		db.Where("AppAlias = ?", c.AppAlias)
	}
	if c.Kind != "" && c.SubconditionValue != "" {
		switch c.Kind {
		case "app":
		case "biz":
		case "interface":
			db.Where("")
		case "service":
			db.Where("ServiceName = ", c.SubconditionValue)
		default:

		}
	}
	db.Where("Timestamp >= ? AND Timestamp <= ?", c.StartTime, c.EndTime)
	db.Where("StatusCode = 'STATUS_CODE_ERROR' AND StatusMessage != '' AND StatusMessage != 'status code:0'")
	db.Group("FiveMinuteInterval")
	if err := db.Order("FiveMinuteInterval, StatusMessageCount Desc").Find(&result).Error; err != nil {
		return errors.Wrap(err, "计算范围时间内的异常失败")
	}
	return nil
}

func (e *Event) EventLists(c *dto.EventStatisticReq, result *[]dto.EventListsResp) error {
	db := e.ChOrm.Table(models.TableNameTrace)
	db.Select([]string{
		"StatusMessage",
		"COUNT(*) AS StatusMessageCount",
		"ServiceName",
	})
	if c.AppAlias != "" {
		db.Where("AppAlias = ?", c.AppAlias)
	}
	if c.Kind != "" && c.SubconditionValue != "" {
		switch c.Kind {
		case "app":
		case "biz":
		case "interface":
			db.Where("")
		case "service":
			db.Where("ServiceName = ", c.SubconditionValue)
		default:

		}
	}
	db.Where("Timestamp >= ? AND Timestamp <= ?", c.StartTime, c.EndTime)
	db.Where("StatusCode = 'STATUS_CODE_ERROR' AND StatusMessage != '' AND StatusMessage != 'status code:0'")
	db.Group("StatusMessage")
	db.Group("ServiceName")
	if err := db.Order("StatusMessageCount DESC;").Find(&result).Error; err != nil {
		return errors.Wrap(err, "统计范围时间内的具体异常失败")
	}
	return nil
}
