package service

import (
	"context"
	"fmt"
	"go-admin/app/observe/models"
	"go-admin/utils"
	"time"
)

type AppAnalyst struct {
	utils.OtService
}

func (e *AppAnalyst) GetAppLatencyListByInterval(ctx context.Context, p *models.AppInterval, l *models.AppLatencyByInterval) error {
	// timestamp 秒级单位
	/*
		计算90分位值
		WITH durations AS (
		    SELECT quantile(0.98)(Duration) AS Duration_90th_percentile
		    FROM otel_traces_duration_ts otdt
		)
	*/

	sql := `WITH durations AS (
		SELECT quantile(?)(Duration) AS Duration_90th_percentile
		FROM otel_traces
		WHERE
		toTimeZone(Timestamp,'Asia/Hong_Kong') >= ? and toTimeZone(Timestamp,'Asia/Hong_Kong') < ? and AppAlias = ?
	    )
	    Select
	    	    toTimeZone(Timestamp,'Asia/Hong_Kong') as ts,
		    round(Duration/1e6, 2) AS Dms,
		    HttpCode
	    FROM otel_traces
	    WHERE
		toTimeZone(Timestamp,'Asia/Hong_Kong') >= ? and toTimeZone(Timestamp,'Asia/Hong_Kong') < ? and AppAlias = ? %s AND (ParentSpanId = '') and Duration >= (SELECT Duration_90th_percentile FROM durations)
	    ORDER BY Timestamp ASC;`

	if p.ServiceName != "" {
		sql = fmt.Sprintf(sql, fmt.Sprintf("AND ServiceName = '%s'", p.ServiceName))
	} else {
		sql = fmt.Sprintf(sql, " ")
	}
	rows, err := e.OlapConn.Query(ctx,
		sql,
		p.Percentile,
		time.Unix(p.StartTime, 0).Format(time.DateTime),
		time.Unix(p.EndTime, 0).Format(time.DateTime),
		p.AppAlias,
		time.Unix(p.StartTime, 0).Format(time.DateTime),
		time.Unix(p.EndTime, 0).Format(time.DateTime),
		p.AppAlias)
	if err != nil {
		e.Log.Errorf("olap error query: rows, %s", err)
		return err
	}
	var t time.Time
	var hc int32
	var d float64
	for rows.Next() {
		if err := rows.Scan(&t, &d, &hc); err != nil {
			e.Log.Errorf("olap error query: successRows, %s", err)
			return err
		}
		if hc < 400 {
			l.Success = append(l.Success, [2]any{t, d})
		} else {
			l.Failed = append(l.Failed, [2]any{t, d})
		}
	}
	rows.Close()
	return nil
}

// for rows.Next() {
// 	err = rows.Scan(&t, &d, &e, &r)
// 	if err != nil {
// 		s.Log.Errorf("扫描行到变量失败：%s", err)
// 		break
// 	}
// 	estr := "success"
// 	if e {
// 		estr = "error"
// 	}
// 	if _, ok := (*result)[r]; !ok {
// 		(*result)[r] = map[string][]models.CoordinatePoint{
// 			"success": {},
// 			"error":   {},
// 		}
// 	}
// 	(*result)[r][estr] = append((*result)[r][estr], [2]any{t, d})
// }
// return

func (e *AppAnalyst) GetServiceLatencyListByInterval(ctx context.Context, p *models.AppServiceInterval, l *models.AppLatencyByInterval) error {
	// timestamp 秒级单位
	/*
		计算90分位值
		WITH durations AS (
		    SELECT quantile(0.98)(Duration) AS Duration_90th_percentile
		    FROM otel_traces_duration_ts otdt
		)
	*/

	sql := `WITH durations AS (
		SELECT quantile(?)(Duration) AS Duration_90th_percentile
		FROM otel_traces
		WHERE
		toTimeZone(Timestamp,'Asia/Hong_Kong') >= ? 
		and toTimeZone(Timestamp,'Asia/Hong_Kong') < ? 
		and AppAlias = ? 
		and ServiceName = ?
	    )
	    Select
	    	    toTimeZone(Timestamp,'Asia/Hong_Kong') as ts,
		    round(Duration/1e6, 2) AS Dms,
		    HttpCode
	    FROM otel_traces
	    WHERE
		toTimeZone(Timestamp,'Asia/Hong_Kong') >= ? 
		and toTimeZone(Timestamp,'Asia/Hong_Kong') < ? 
		and AppAlias = ? 
		AND ServiceName = ?
		and Duration >= (SELECT Duration_90th_percentile FROM durations)
	    ORDER BY Timestamp ASC;`

	rows, err := e.OlapConn.Query(ctx,
		sql,
		p.Percentile,
		time.Unix(p.StartTime, 0).Format(time.DateTime),
		time.Unix(p.EndTime, 0).Format(time.DateTime),
		p.AppAlias,
		p.SourceService,
		time.Unix(p.StartTime, 0).Format(time.DateTime),
		time.Unix(p.EndTime, 0).Format(time.DateTime),
		p.AppAlias,
		p.SourceService)
	if err != nil {
		e.Log.Errorf("olap error query: rows, %s", err)
		return err
	}
	var t time.Time
	var hc int32
	var d float64
	for rows.Next() {
		if err := rows.Scan(&t, &d, &hc); err != nil {
			e.Log.Errorf("olap error query: successRows, %s", err)
			return err
		}
		if hc < 400 {
			l.Success = append(l.Success, [2]any{t, d})
		} else {
			l.Failed = append(l.Failed, [2]any{t, d})
		}
	}
	rows.Close()
	return nil
}

func (e *AppAnalyst) GetEdgeLatencyListByInterval(ctx context.Context, p *models.AppServiceInterval, l *models.AppLatencyByInterval) error {
	// timestamp 秒级单位
	/*
		计算90分位值
		WITH durations AS (
		    SELECT quantile(0.98)(Duration) AS Duration_90th_percentile
		    FROM otel_traces_duration_ts otdt
		)
	*/

	sql := `WITH ? AS StartTime, ? AS EndTime, ? AS appAlias,
	durations AS (
			SELECT quantile(?)(Duration) AS Duration_90th_percentile
			FROM
			(
			SELECT
				TraceId,
				SpanId,
				ParentSpanId,
				ServiceName,
				Duration,
				StatusCode,
				AppAlias,
				HttpCode
			FROM otel.otel_traces
			WHERE AppAlias=appAlias AND Timestamp > StartTime AND Timestamp < EndTime AND ServiceName = ?
		) AS ot1
		INNER JOIN
		(
			SELECT
				SpanId,
				ServiceName
			FROM otel.otel_traces
			WHERE AppAlias=appAlias and Timestamp > StartTime AND Timestamp < EndTime AND ServiceName = ?
		) AS ot2 ON ot2.SpanId = ot1.ParentSpanId
	)
	Select
		    toTimeZone(Timestamp,'Asia/Hong_Kong') as ts,
			round(Duration/1e6, 2) AS Dms,
			HttpCode
	FROM
	(
			SELECT
				Timestamp,
				TraceId,
				SpanId,
				ParentSpanId,
				ServiceName,
				Duration,
				StatusCode,
				AppAlias,
				HttpCode
			FROM otel.otel_traces
			WHERE AppAlias=appAlias AND Timestamp > StartTime AND Timestamp < EndTime AND ServiceName = ?
		) AS ot1
		INNER JOIN
		(
			SELECT
				SpanId,
				ServiceName
			FROM otel.otel_traces
			WHERE AppAlias=appAlias and Timestamp > StartTime AND Timestamp < EndTime AND ServiceName = ?
		) AS ot2 ON ot2.SpanId = ot1.ParentSpanId
	WHERE 
	Duration >= (SELECT Duration_90th_percentile FROM durations)
	ORDER BY Timestamp ASC;`

	rows, err := e.OlapConn.Query(ctx,
		sql,
		p.StartTime,
		p.EndTime,
		p.AppAlias,
		p.Percentile,
		p.TargetService,
		p.SourceService,
		p.TargetService,
		p.SourceService,
	)
	if err != nil {
		e.Log.Errorf("olap error query: rows, %s", err)
		return err
	}
	var t time.Time
	var hc int32
	var d float64
	for rows.Next() {
		if err := rows.Scan(&t, &d, &hc); err != nil {
			e.Log.Errorf("olap error query: successRows, %s", err)
			return err
		}
		if hc < 400 {
			l.Success = append(l.Success, [2]any{t, d})
		} else {
			l.Failed = append(l.Failed, [2]any{t, d})
		}
	}
	rows.Close()
	return nil
}
