package alert

import (
	"fmt"
	"log"
	"os"
	"os/signal"
	"time"

	"github.com/go-admin-team/go-admin-core/config/source/file"
	"github.com/go-admin-team/go-admin-core/sdk/config"
	"github.com/go-admin-team/go-admin-core/sdk/pkg"
	"github.com/spf13/cobra"

	"go-admin/common/database"
	extConfig "go-admin/config"
	"go-admin/handler"
)

var (
	configYml string
	apiCheck  bool
	StartCmd  = &cobra.Command{
		Use:          "alert",
		Short:        "Start Alert server",
		Example:      "go-admin alert -c config/settings.yml",
		SilenceUsage: true,
		PreRun: func(cmd *cobra.Command, args []string) {
			setup()
		},
		RunE: func(cmd *cobra.Command, args []string) error {
			return run()
		},
	}
)

func init() {
	StartCmd.PersistentFlags().StringVarP(&configYml, "config", "c", "config/settings.yml", "Start server with provided configuration file")
}

func setup() {
	config.ExtendConfig = &extConfig.ExtConfig
	//1. 读取配置
	config.Setup(
		file.NewSource(file.WithPath(configYml)),
		database.Setup,
		// storage.Setup,
		// olap.Setup,
	)
	usageStr := `starting alert server...`
	log.Println(usageStr)
}

func run() error {
	handler.SetAlertManager()
	if config.ApplicationConfig.Mode == pkg.ModeProd.String() {
		// gin.SetMode(gin.ReleaseMode)

	}

	//TODO:10s一次的检测，设定可配置
	// productor
	ticker := time.Tick(time.Duration(handler.GetAlertManager().Interval) * time.Second)
	go func() {
		handler.GetAlertManager().PolicyProducter()
		for {
			select {
			case <-ticker:
				fmt.Println("alert policy producting....")
				handler.GetAlertManager().PolicyProducter()
			}
		}
	}()

	go func() {
		handler.GetAlertManager().PolicyConsumer()
	}()

	defer handler.CloseAlertManager()
	fmt.Println(pkg.Red("Start alert Service"))
	quit := make(chan os.Signal)
	signal.Notify(quit, os.Interrupt)
	<-quit

	// ctx, cancel := context.WithTimeout(context.Background(), 5*time.Second)
	// defer cancel()
	fmt.Printf("%s Shutdown alert server ... \r\n", pkg.GetCurrentTimeStr())
	log.Println("Alert server exiting")
	return nil
}
